@@ -1,17 +1,25 @@
 from __future__ import absolute_import
 from __future__ import unicode_literals
 
+import sys
+if sys.version_info < (3, 3):
+    from pipes import quote as _quote
+else:
+    from shlex import quote as _quote
 import os.path
-import pipes
 import subprocess
 import sys
 import sysconfig
+import platform
 
-import mock
+try:
+    from unittest import mock
+except ImportError:
+    import mock
 import pytest
 
 import nodeenv
-
+from nodeenv import IncompleteRead
 
 HERE = os.path.abspath(os.path.dirname(__file__))
 
@@ -25,7 +33,7 @@ def test_smoke(tmpdir):
         '-m', 'nodeenv', '--prebuilt', nenv_path,
     ])
     assert os.path.exists(nenv_path)
-    activate = pipes.quote(os.path.join(nenv_path, 'bin', 'activate'))
+    activate = _quote(os.path.join(nenv_path, 'bin', 'activate'))
     subprocess.check_call([
         'sh', '-c', '. {} && node --version'.format(activate),
     ])
@@ -40,13 +48,13 @@ def test_smoke_n_system_special_chars(tmpdir):
         '-m', 'nodeenv', '-n', 'system', nenv_path,
     ))
     assert os.path.exists(nenv_path)
-    activate = pipes.quote(os.path.join(nenv_path, 'bin', 'activate'))
+    activate = _quote(os.path.join(nenv_path, 'bin', 'activate'))
     subprocess.check_call([
         'sh', '-c', '. {} && node --version'.format(activate),
     ])
 
 
-@pytest.yield_fixture
+@pytest.fixture
 def mock_index_json():
     # retrieved 2019-12-31
     with open(os.path.join(HERE, 'nodejs_index.json'), 'rb') as f:
@@ -54,7 +62,7 @@ def mock_index_json():
             yield
 
 
-@pytest.yield_fixture
+@pytest.fixture
 def cap_logging_info():
     with mock.patch.object(nodeenv.logger, 'info') as mck:
         yield mck
@@ -112,6 +120,8 @@ def test_mirror_option():
     # Check if running on musl system and delete last mirror if it is
     if sys_type in musl_type:
         urls.pop()
+    elif platform.machine() == "riscv64":
+        urls.pop()
     with open(os.path.join(HERE, 'nodejs_index.json'), 'rb') as f:
         def rewind(_):
             f.seek(0)
@@ -123,9 +133,9 @@ def test_mirror_option():
             else:
                 test_argv = argv
             with mock.patch.object(sys, 'argv', test_argv), \
-                 mock.patch.object(nodeenv.logger, 'info') as mock_logger, \
-                 mock.patch.object(nodeenv, 'urlopen',
-                                   side_effect=rewind) as mock_urlopen:
+                mock.patch.object(nodeenv.logger, 'info') as mock_logger, \
+                mock.patch.object(nodeenv, 'urlopen',
+                                  side_effect=rewind) as mock_urlopen:
                 nodeenv.src_base_url = None
                 nodeenv.main()
                 mock_urlopen.assert_called_with(url)
@@ -140,3 +150,14 @@ def test_get_latest_node_version():
 @pytest.mark.usefixtures('mock_index_json')
 def test_get_lts_node_version():
     assert nodeenv.get_last_lts_node_version() == '12.14.0'
+
+
+def test__download_node_file():
+    with mock.patch.object(nodeenv, 'urlopen') as m_urlopen:
+        m_urlopen.side_effect = IncompleteRead("dummy")
+        with pytest.raises(IncompleteRead):
+            nodeenv._download_node_file(
+                "https://dummy/nodejs.tar.gz",
+                n_attempt=5
+            )
+        assert m_urlopen.call_count == 5
